<?php

use WHMCS_DirectAdmin_SiteBuilder_Module\DirectadminApiInterface;

require_once __DIR__.'/DirectadminApiInterface.php';
require_once __DIR__.'/httpsocket-3.0.1.php';

class SiteproInternalApi implements DirectadminApiInterface {
	/**
	 * DA API comunication socket
	 * @var \HTTPSocket 
	 */	
	private $socket;

	/** @var string */
	private $user;

	/** @var string|null */
	private $password = null;

	public function __construct($host, $user, $pass = null, $port = 2222, $proto = 'http') {
		$this->user = $user;
		$this->socket = new HTTPSocket();
		$this->socket->add_header('Host', (($port == 80) ? $host : "$host:$port"));
		// Note: prioritize loopback IP for API on https (or maybe always)
		// to avoid "could not connect to host" errors on install.
		$apiHost = (($proto == 'https') ? 'ssl://' : '')
			.(($proto != 'https' && @gethostbyname($host) != $host) ? $host : '127.0.0.1');
		$this->socket->connect($apiHost, $port);
		$this->socket->set_login($this->user, $pass);
	}

	/** @return string|null */
	public function lastApiCallError() {
		return empty($this->socket->error) ? null : implode("\n", $this->socket->error);
	}

	/** @param string $password */
	public function setPassword($password) {
		$this->password = $password;
		$this->socket->set_login($this->user, $password);
	}

	public function getPassword() {
		return $this->password;
	}

	/**
	 * Call DA API
	 * @param string $command API command to call
	 * @param array $args associative array with command arguments
	 * @param bool $raw if true unparsed result will be returned
	 * @return mixed command result
	 */
	public function call($command, $args = array(), $raw = false) {
		$this->socket->query('/'.$command, $args);
		$result = $raw ? $this->socket->fetch_body() : $this->socket->fetch_parsed_body();
		if ($result) {
			return $result;
		}
		return null;
	}
}
