<?php

namespace WHMCS_DirectAdmin_SiteBuilder_Module;

use ErrorException;

require_once __DIR__.'/SiteproApiClient.php';

class SiteproModule {
	public static $apiUrl = '';
	public static $apiUser = '';
	public static $apiPass = '';
	public static $userId = null;
	public static $licenseHash = null;
	private static $panel = 'DirectAdmin';
	private static $createFromHash = null;

	public static function setup($apiUrl, $apiUser, $apiPass, $userId = null, $licenseHash = null, $panel = 'DirectAdmin', $createFromHash = null) {
		self::$apiUrl = $apiUrl;
		self::$apiUser = $apiUser;
		self::$apiPass = $apiPass;
		self::$userId = $userId;
		self::$licenseHash = $licenseHash;
		self::$panel = $panel;
		self::$createFromHash = $createFromHash;
	}

	/** @var array|null */
	private $subdomains = null;
	
	/** @var string|null */
	private $domain = null;
	
	/** @var string|null */
	private $user = null;

	/** @var string|null */
	private $hostingPlan = null;

	/** @var array|null */
	private $userConf = null;
	
	/** @var string|null */
	private $baseUrl = null;

	/** @var string|null */
	private $host = null;

	/** @var string */
	private $proto = 'http';

	/** @var int */
	private $port = 2222;

	/** @var string */
	private $baseUri = '/CMD_PLUGINS/siteprobuilder/index2.raw';

	/** @var string|null */
	private $loginKey = null;

	/** @var DirectadminApiInterface|null */
	private $api = null;

	/** @return DirectadminApiInterface */
	public function &getApi() {
		return $this->api;
	}

	public function setApi(DirectadminApiInterface $api) {
		$this->api = $api;
	}

	public function &getLoginKey() {
		return $this->loginKey;
	}

	public function setLoginKey($loginKey) {
		$this->loginKey = $loginKey;
	}

	/** @param int $port */
	public function setPort($port) {
		if (($t = intval($port))) $this->port = $t;
	}

	/** @param string $proto */
	public function setProto($proto) {
		if ((preg_match('#^http|https$#i', $proto))) {
			$this->proto = strtolower($proto);
		}
	}

	/** @param string $host */
	public function setHost($host) {
		$this->host = $host;
	}

	/**
	 * @param string|null $subdomain
	 * @throws ErrorException
	 * @return string
	 */
	public function getBuilderUrl($subdomain = null) {
		$domain = $this->domain;
		$qParams = array('d' => $domain);
		if ($subdomain) {
			foreach ($this->getSubdomains($domain) as $li) {
				if ($li->id == $subdomain || $li->name == $subdomain) {
					$qParams['sd'] = $li->id;
					$domain = $li->name;
					break;
				}
			}
		}
		$apiUrl = $this->getBaseUrl($qParams).'api/';
		$api = new SiteproApiClient(self::$apiUrl, self::$apiUser, self::$apiPass);
		$usr = $api->requestLogin($domain, "admin|{$this->user}", $this->getLoginKey(), $apiUrl, $this->getHostingPlan(), self::$panel, self::$createFromHash);
		if (is_object($usr) && isset($usr->url) && $usr->url) {
			return $usr->url;
		} else if (is_object($usr) && isset($usr->error) && $usr->error) {
			throw new ErrorException('Error: '.$usr->error->message);
		} else {
			throw new ErrorException('Error: server error');
		}
	}
	
	public function isEnabled() {
		$uc = $this->getUserConf();
		return (!isset($uc['siteprobuilder']) || $uc['siteprobuilder'] == 'ON' || $uc['siteprobuilder'] == 1 || $uc['siteprobuilder'] == 'yes');
	}

	public function deleteLoginKey($name) {
		$this->callApi('CMD_API_LOGIN_KEYS', array(
			'action' => 'select',
			'delete' => 'Delete',
			'select0' => $name,
		));
	}

	/**
	 * @param string $name
	 * @return string
	 * @throws ErrorException
	 */
	public function createLoginKey($name = 'siteprobuilder') {
		$key = md5(microtime());
		$this->deleteLoginKey($name);
		$this->callApi('CMD_API_LOGIN_KEYS', array(
			'action' => 'create',
			'clear_key' => 'yes',
			'allow_htm' => 'yes',
			'never_expires' => 'yes',
			'max_uses' => 0,
			'year' => date('Y'),
			'month' => date('m'),
			'day' => date('d'),
			'hour' => date('H'),
			'minute' => date('i'),
			'ips' => '',
			'key' => $key,
			'key2' => $key,
			'keyname' => $name,
			'passwd' => $this->api->getPassword(),
			'select_allow184' => 'CMD_PLUGINS',
			'select_allow185' => 'CMD_LOGIN'
		));
		$this->callApi('CMD_API_LOGIN_KEYS', array(
			'action' => 'modify',
			'clear_key' => 'yes',
			'allow_htm' => 'yes',
			'never_expires' => 'yes',
			'max_uses' => 0,
			'year' => date('Y'),
			'month' => date('m'),
			'day' => date('d'),
			'hour' => date('H'),
			'minute' => date('i'),
			'ips' => '',
			'key' => $key,
			'key2' => $key,
			'keyname' => $name,
			'passwd' => $this->api->getPassword(),
			'select_allow184' => 'CMD_PLUGINS',
			'select_allow185' => 'CMD_LOGIN'
		));
		return $key;
	}
	
	/**
	 * Call DA API
	 * @param string $command API command to call
	 * @param array $args associative array with command arguments
	 * @param bool $raw if true unparsed result will be returned
	 * @return mixed command result
	 * @throws ErrorException
	 */
	public function callApi($command, $args = array(), $raw = false) {
		try {
			return $this->api->call($command, $args, $raw);
		} catch (ErrorException $ex) {
			throw new ErrorException('API call error: '.$ex->getMessage());
		}
	}
	
	/** @param string $user */
	public function setUser($user) {
		$this->user = $user;
	}
	
	public function setDomain($domain) {
		$this->domain = $domain;
	}

	/**
	 * Get subdomain list for a domain
	 * @param string $domain
	 * @return array
	 */
	public function getSubdomains($domain) {
		if (is_null($this->subdomains)) {
			$this->subdomains = array();
			$result = $this->callApi('CMD_API_SUBDOMAINS', array('domain' => $domain));
			if ($result && isset($result['list']) && is_array($result['list'])) {
				$this->subdomains[$domain] = array();
				foreach ($result['list'] as $li) { $this->subdomains[$domain][] = (object) array('id' => $li, 'name' => $li.'.'.$domain); }
			}
		}
		return isset($this->subdomains[$domain]) ? $this->subdomains[$domain] : array();
	}

	/** @param string $uri */
	public function setBaseUri($uri) {
		$this->baseUri = $uri;
	}
	
	public function getBaseUrl($params = null) {
		if (is_null($this->baseUrl)) {
			$this->baseUrl = $this->proto.'://'
				.$this->host . ($this->port == 80 ? "" : ":{$this->port}")
				.$this->baseUri . '?route=/boot.php/';
		}
		$baseUrl = $this->baseUrl;
		if ($params && is_array($params)) {
			$prm = array();
			foreach ($params as $k => $v) { $prm[] = urlencode($k).'='.urldecode($v);  }
			if (!empty($prm)) {
				$url = explode('?', $baseUrl, 2);
				if (isset($url[1])) $prm[] = $url[1];
				$baseUrl = $url[0].'?'.implode('&', $prm);
			}
		}	
		return $baseUrl;
	}

	private function getUserConf() {
		if (is_null($this->userConf)) {
			$this->userConf = $this->callApi('CMD_API_SHOW_USER_CONFIG', array('user' => $this->user));
		}
		return $this->userConf;
	}
	
	public function getHostingPlan() {
		if (is_null($this->hostingPlan)) {
			$uc = $this->getUserConf();
			$this->hostingPlan = (isset($uc['package']) && $uc['package']) ? $uc['package'] : null;
			if ((!$this->hostingPlan || $this->hostingPlan == 'custom') && isset($uc['original_package']) && $uc['original_package']) {
				$this->hostingPlan = $uc['original_package'];
			}
		}
		return $this->hostingPlan;
	}

	/** @param string $hostingPlan */
	public function setHostingPlan($hostingPlan) {
		$this->hostingPlan = $hostingPlan;
	}
}
